'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const pref_deck_suit_1 = require("./enums/pref.deck.suit");
const normalize_1 = require("./functions/card/normalize");
const get_value_label_1 = require("./functions/card/get.value.label");
const convert_ppn_to_card_1 = require("./functions/card/convert.ppn.to.card");
const convert_club_card_to_ppn_1 = require("./functions/card/convert.club.card.to.ppn");
const convert_spade_card_to_ppn_1 = require("./functions/card/convert.spade.card.to.ppn");
const convert_heart_card_to_ppn_1 = require("./functions/card/convert.heart.card.to.ppn");
const convert_diamond_card_to_ppn_1 = require("./functions/card/convert.diamond.card.to.ppn");
class PrefDeckCard {
    constructor(suit, value) {
        this._suit = suit;
        this._value = value;
        this._label = PrefDeckCard.valueLabel(this._value) + '' + this._suit;
        this._unicode = PrefDeckCard.valueLabel(this._value) + '' + PrefDeckCard.suitToUnicode(this._suit);
        this._ppn = PrefDeckCard.cardToPPN(this);
    }
    static valueLabel(value) {
        return (0, get_value_label_1.default)(value);
    }
    static compare(c1, c2, t) {
        if (c1.suit === c2.suit)
            return (0, normalize_1.default)(c2.value - c1.value);
        if (t && c2.suit === t)
            return 1;
        return -1;
    }
    static winner(c1, c2, t) {
        return PrefDeckCard.compare(c1, c2, t) < 0 ? c1 : c2;
    }
    static suitToUnicode(suit) {
        switch (suit) {
            case pref_deck_suit_1.default.SPADE:
                return '♠';
            case pref_deck_suit_1.default.DIAMOND:
                return '♦';
            case pref_deck_suit_1.default.HEART:
                return '♥';
            case pref_deck_suit_1.default.CLUB:
                return '♣';
        }
    }
    static cardToPPN(c) {
        switch (c.suit) {
            case pref_deck_suit_1.default.SPADE:
                return (0, convert_spade_card_to_ppn_1.default)(c.value);
            case pref_deck_suit_1.default.DIAMOND:
                return (0, convert_diamond_card_to_ppn_1.default)(c.value);
            case pref_deck_suit_1.default.HEART:
                return (0, convert_heart_card_to_ppn_1.default)(c.value);
            case pref_deck_suit_1.default.CLUB:
                return (0, convert_club_card_to_ppn_1.default)(c.value);
        }
    }
    static ppnToCard(ppn) {
        return (0, convert_ppn_to_card_1.default)(ppn);
    }
    beats(c, t) {
        return PrefDeckCard.compare(this, c, t) < 0;
    }
    get suit() {
        return this._suit;
    }
    get value() {
        return this._value;
    }
    get label() {
        return this._label;
    }
    get unicode() {
        return this._unicode;
    }
    get ppn() {
        return this._ppn;
    }
}
exports.default = PrefDeckCard;
//# sourceMappingURL=pref.deck.card.js.map