'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PrefDeckSorting = exports.PrefDeckValue = exports.PrefDeckSuit = exports.PrefDeckTrick = exports.PrefDeckPile = exports.PrefDeckCard = void 0;
const _ = require("lodash");
const pref_deck_card_1 = require("./pref.deck.card");
exports.PrefDeckCard = pref_deck_card_1.default;
const pref_deck_pile_1 = require("./pref.deck.pile");
exports.PrefDeckPile = pref_deck_pile_1.default;
const pref_deck_trick_1 = require("./pref.deck.trick");
exports.PrefDeckTrick = pref_deck_trick_1.default;
const pref_deck_suit_1 = require("./enums/pref.deck.suit");
exports.PrefDeckSuit = pref_deck_suit_1.default;
const pref_deck_value_1 = require("./enums/pref.deck.value");
exports.PrefDeckValue = pref_deck_value_1.default;
const pref_deck_sorting_1 = require("./enums/pref.deck.sorting");
exports.PrefDeckSorting = pref_deck_sorting_1.default;
const is_valid_deck_1 = require("./functions/deck/is.valid.deck");
const random_range_1 = require("./functions/deck/random.range");
const weighted_range_1 = require("./functions/deck/weighted.range");
const human_shuffle_1 = require("./functions/deck/human.shuffle");
const simple_shuffle_1 = require("./functions/deck/simple.shuffle");
const control_deck_1 = require("./functions/deck/control.deck");
const weighted_cuts_1 = require("./functions/deck/weighted.cuts");
class PrefDeck extends pref_deck_pile_1.default {
    constructor() {
        super(control_deck_1.default);
        return this;
    }
    restore(cards) {
        if (!(0, is_valid_deck_1.default)(cards))
            throw new Error('Deck::restore:Invalid cards to restore from: ' + _.size(cards) + ' ' + new pref_deck_pile_1.default(cards).unicode);
        this._cards = _.clone(cards);
        return this;
    }
    get cut() {
        this._cards = this._cards.concat(this._cards.splice(0, _.sample(weighted_cuts_1.default)));
        return this;
    }
    get shuffle() {
        _.each((0, random_range_1.default)(), () => (this._cards = (0, human_shuffle_1.default)(this._cards)));
        _.each((0, weighted_range_1.default)(), () => (this._cards = (0, simple_shuffle_1.default)(this._cards)));
        return this.cut;
    }
    get valid() {
        return (0, is_valid_deck_1.default)(this.cards);
    }
    get deal() {
        const hand1a = _.slice(this._cards, 0, 5);
        const hand2a = _.slice(this._cards, 5, 10);
        const hand3a = _.slice(this._cards, 10, 15);
        const talon = _.slice(this._cards, 15, 17);
        const hand1b = _.slice(this._cards, 17, 22);
        const hand2b = _.slice(this._cards, 22, 27);
        const hand3b = _.slice(this._cards, 27, 32);
        return {
            hand1: new pref_deck_pile_1.default(_.concat(hand1a, hand1b)),
            hand2: new pref_deck_pile_1.default(_.concat(hand2a, hand2b)),
            hand3: new pref_deck_pile_1.default(_.concat(hand3a, hand3b)),
            talon: { talon1: talon[0], talon2: talon[1] }
        };
    }
}
exports.default = PrefDeck;
//# sourceMappingURL=pref.deck.js.map