'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const pref_deck_sorting_1 = require("./enums/pref.deck.sorting");
const sort_cards_1 = require("./functions/pile/sort.cards");
const sort_by_suits_1 = require("./functions/pile/sort.by.suits");
const is_suit_sorting_1 = require("./functions/pile/is.suit.sorting");
const is_reverse_sorting_1 = require("./functions/pile/is.reverse.sorting");
const convert_cards_to_ppns_1 = require("./functions/pile/convert.cards.to.ppns");
const convert_cards_to_labels_1 = require("./functions/pile/convert.cards.to.labels");
const convert_cards_to_unicodes_1 = require("./functions/pile/convert.cards.to.unicodes");
class PrefDeckPile {
    constructor(cards) {
        this._cards = [];
        this._original = [];
        _.each(cards, (card) => {
            this._cards.push(card);
            this._original.push(card);
        });
    }
    sort(sorting = pref_deck_sorting_1.default.BLACK) {
        if (sorting === pref_deck_sorting_1.default.NONE)
            return this;
        if ((0, is_suit_sorting_1.default)(sorting)) {
            const reverse = (0, is_reverse_sorting_1.default)(sorting);
            this._cards = (0, sort_by_suits_1.default)(this._cards, reverse);
            return this;
        }
        this._cards = (0, sort_cards_1.default)(this._cards, sorting);
        return this;
    }
    get cards() {
        return this._cards;
    }
    get label() {
        return (0, convert_cards_to_labels_1.default)(this._cards);
    }
    get unicode() {
        return (0, convert_cards_to_unicodes_1.default)(this._cards);
    }
    get ppn() {
        return (0, convert_cards_to_ppns_1.default)(this._cards);
    }
    get original() {
        return this._original;
    }
    get originalLabel() {
        return (0, convert_cards_to_labels_1.default)(this._original);
    }
    get originalUnicode() {
        return (0, convert_cards_to_unicodes_1.default)(this._original);
    }
    get originalPPN() {
        return (0, convert_cards_to_ppns_1.default)(this._original);
    }
}
exports.default = PrefDeckPile;
//# sourceMappingURL=pref.deck.pile.js.map