'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const pref_deck_trump_1 = require("./enums/pref.deck.trump");
const player_to_json_1 = require("./functions/trick/player.to.json");
const convert_trump_to_suit_1 = require("./functions/trick/convert.trump.to.suit");
const convert_suit_to_trump_1 = require("./functions/trick/convert.suit.to.trump");
const is_first_player_the_winner_1 = require("./functions/trick/is.first.player.the.winner");
const is_second_player_the_winner_1 = require("./functions/trick/is.second.player.the.winner");
class PrefDeckTrick {
    constructor(players, trumpSuit) {
        this._trump = pref_deck_trump_1.default.NONE;
        this._winner = undefined;
        this._players = players;
        if (trumpSuit !== undefined)
            this._trump = (0, convert_suit_to_trump_1.default)(trumpSuit);
    }
    throw(player, card) {
        if (!this._first)
            this._first = { player, card };
        else if (!this._second)
            this._second = { player, card };
        else if (this._players === 3 && !this._third)
            this._third = { player, card };
        else
            throw new Error('PrefDeckTrick::throw:Trick is already full:[' + this.string + ']');
        this._calculateWinner();
        return this;
    }
    get first() {
        return this._first ? this._first.card : null;
    }
    get second() {
        return this._second ? this._second.card : null;
    }
    get third() {
        return this._third ? this._third.card : null;
    }
    get suit() {
        return (0, convert_trump_to_suit_1.default)(this._trump);
    }
    get winner() {
        if (!this._winner)
            this._calculateWinner();
        if (this._winner)
            return this._winner;
        throw new Error('PrefDeckTrick::winner:Winner not found: [' + this.string + ']');
    }
    get ppn() {
        const a = this._first ? this._first.card.ppn : '';
        const b = this._second ? this._second.card.ppn : '';
        const c = this._third ? this._third.card.ppn : '';
        return a + b + c;
    }
    get json() {
        return {
            players: this._players,
            first: (0, player_to_json_1.default)(this._first),
            second: (0, player_to_json_1.default)(this._second),
            third: (0, player_to_json_1.default)(this._third),
            trump: this._trump,
            winner: this._winner
        };
    }
    get string() {
        return JSON.stringify(this.json);
    }
    get full() {
        let cnt = 0;
        if (this._first)
            cnt++;
        if (this._second)
            cnt++;
        if (this._third)
            cnt++;
        return cnt === this._players;
    }
    _calculateWinner() {
        this._winner = undefined;
        if (!this.full)
            return this;
        const suit = (0, convert_trump_to_suit_1.default)(this._trump);
        const c1 = this._first.card;
        const c2 = this._second.card;
        const c3 = this._third ? this._third.card : undefined;
        if ((0, is_first_player_the_winner_1.default)(c1, c2, c3, suit))
            this._winner = this._first.player;
        else if ((0, is_second_player_the_winner_1.default)(c1, c2, c3, suit))
            this._winner = this._second.player;
        else
            this._winner = this._third.player;
        return this;
    }
}
exports.default = PrefDeckTrick;
//# sourceMappingURL=pref.deck.trick.js.map